#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/vbox.sh - configuration generator script for VBox
#
# Copyright (c) 2002-2016 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2002-02-26  jed
# Last Update: $Id: vbox.sh 43659 2016-10-08 19:08:45Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib
. /var/install/include/jedlib

module_name='vbox'

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2> /tmp/${module_name}-config-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

pgmname=`basename $0`

### set pathes ###
vbox_dir=/usr/local/vbox
vbox_spooldir=/var/spool/vbox
tmpdir=/tmp

### set file names ###
apache2file=/etc/config.d/apache2
basefile=/etc/config.d/base
mailfile=/etc/config.d/mail
vboxfile=/etc/config.d/vbox
vmailfile=/etc/config.d/vmail
ssmtpfile=/etc/config.d/ssmtp
packagefile=/var/install/packages/vbox

generate_vbox=""                                                    # will be set later
generate_vboxd=${vbox_dir}/etc/vboxd.conf
generate_vboxgetty=${vbox_dir}/etc/vboxgetty.conf
generate_xinet_vbox=/etc/xinetd.d/vbox
generate_logrotate_vbox=/etc/logrotate.d/vbox
generate_aliases=/etc/aliases.vbox
generate_services=/etc/services.vbox
generate_inittab=/etc/inittab.vbox
generate_userlist=${vbox_spooldir}/vbox.users
configlog_file=${vbox_spooldir}/vbox-configlog

# other parameters
vbox_version="v`grep "<version>" ${packagefile} | sed 's#<version>\(.*\)</version>#\1#'`"

### load previous user list ###
if [ -f ${generate_userlist} ]
then
    ### load previous configuration ###
    . ${generate_userlist}
else
    ### no previous user list ###
    VBOX_OLD_USER_N='0'
fi

### load configuration ###
. ${basefile}

if [ -f ${vboxfile} ]
then
    . ${vboxfile}
else
    mecho --error "configuration file '${vboxfile}' not found!"
    exit 1
fi

#----------------------------------------------------------------------------------------
# check if apache2 has been enabled
#----------------------------------------------------------------------------------------
check_installed_apache2 ()
{
    retval=1

    if [ -f ${apache2file} ]
    then
        # apache2 installed
        . ${apache2file}

        if [ "${START_APACHE2}" = "yes" ]
        then
            # mail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "apache2 has been enabled ..."
            fi
            retval=0
        else
            # apache2 deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho --warn "apache2 has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check if mail has been enabled
#----------------------------------------------------------------------------------------
check_installed_mail ()
{
    retval=1

    if [ -f ${mailfile} ]
    then
        # mail installed
        . ${mailfile}

        if [ "${START_MAIL}" = "yes" ]
        then
            # mail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "mail has been enabled ..."
            fi
            retval=0
        else
            # mail deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho --warn "mail has been disabled ..."
            fi
        fi
    fi

    return $retval
}

#----------------------------------------------------------------------------------------
# check if vmail has been enabled
#----------------------------------------------------------------------------------------
check_installed_vmail ()
{
    retval=1

    if [ -f ${vmailfile} ]
    then
        # mail installed
        . ${vmailfile}

        if [ "${START_VMAIL}" = "yes" ]
        then
            # mail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "vmail has been enabled ..."
            fi
            retval=0
        else
            # mail deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho --warn "vmail has been disabled ..."
            fi
        fi
    fi

    return $retval
}

#----------------------------------------------------------------------------------------
# check if ssmtp has been enabled
#----------------------------------------------------------------------------------------
check_installed_ssmtp ()
{
    retval=1

    if [ -f ${ssmtpfile} ]
    then
        # ssmtp activated
        if [ "${1}" != "-quiet" ]
        then
            mecho "ssmtp is always enabled ..."
        fi
        retval=0
    fi

    return $retval
}

#----------------------------------------------------------------------------------------
# check time parameter
# input:  $1 - time string
#         $2 - parameter name
#----------------------------------------------------------------------------------------
check_time_setting ()
{
#exec 2> ./vbox-trace-$$.test
#set -x
    time_str="$1"
    param_name="$2"

    if [ "$time_str" != "*" ]
    then
        # check for valid timespan(s)
        exitflag=0
        while [ $exitflag -eq 0 ]
        do
            echo $time_str | grep -q ","

            if [ $? -eq 0 ]
            then
                time_span=`echo $time_str | cut -d, -f1`
                time_str=`echo $time_str | cut -d, -f2-`
            else
                time_span="$time_str"
                exitflag=1
            fi

            if ! check_timespan "$time_span"
            then
                # error
                write_to_config_log -warn "Invalid timespan '$time_span' given, please check $param_name!"
            fi
        done
    fi
#set +x
}

#----------------------------------------------------------------------------------------
# check if active user exists
# return: 0 - active user found
#         1 - no active user found
#----------------------------------------------------------------------------------------
check_active_users ()
{
    retval=1

    idx=1
    while [ ${idx} -le ${VBOX_USER_N} ]
    do
        eval active='$VBOX_USER_'${idx}'_ACTIVE'

        if [ "${active}" = "yes" ]
        then
            # get port
            retval=0
            break
        fi

        idx=`expr ${idx} + 1`
    done

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check timespan
# input:  $1 - timespan (format: 20:15-23:30)
# return: 0  - ok
#         1  - error time1 = time2
#         2  - time1 > time2
#----------------------------------------------------------------------------------------
check_timespan ()
{
    time_span="$1"

    time1=`echo $time_span | cut -d- -f1`
    time2=`echo $time_span | cut -d- -f2`
    sectime1=`date +%s -d "2000-01-01 $time1"`
    sectime2=`date +%s -d "2000-01-01 $time2"`

    if [ $sectime1 -eq $sectime2 ]
    then
        # error: time1 = time2
        rtc=1
    elif [ $sectime1 -gt $sectime2 ]
    then
        # error: time1 > time2
        rtc=2
    else
        # ok: time1 < time2
        rtc=0
    fi

    return $rtc
}

#----------------------------------------------------------------------------------------
# check if less than 3 points are found in ip-address
# and return '.' if yes.
# input:  $1 - ip-address
# return: ''/'.' character string
#----------------------------------------------------------------------------------------
check_3point ()
{
    if [ `echo -n ${1} | sed 's/[0-9]//g' | wc -m` -lt 3 ]
    then
        # return additional point
        echo '.'
    else
        echo ''
    fi
}

#----------------------------------------------------------------------------------------
# create isdn devices
#----------------------------------------------------------------------------------------
create_isdn_devices ()
{
    mecho "checking ISDN devices ..."

    _cid_idx=0
    while [ ${_cid_idx} -le 15 ]
    do
        if [ ! -c /dev/ttyI${_cid_idx} ]
        then
            mknod -m 0644 /dev/ttyI${_cid_idx} c 43 ${_cid_idx}
        fi

        _cid_idx=`expr ${_cid_idx} + 1`
    done
}

#----------------------------------------------------------------------------------------
# add vboxd ports to /etc/services
#----------------------------------------------------------------------------------------
add_ports_to_services ()
{
    mecho "creating services file ..."

    {
        echo "#"
        echo "# $generate_services list file generated by $pgmname $vbox_version"
        echo "#"
        echo "vboxd          20012/tcp"
        echo "vboxd          20012/udp"
    } > $generate_services

    /var/install/bin/update-services $pgmname
}

#----------------------------------------------------------------------------------------
# / etc/aliases
#----------------------------------------------------------------------------------------
generate_smtp_aliases ()
{
    mecho "creating aliases ..."

    {
        echo "#"
        echo "# $generate_aliases list file generated by $pgmname $vbox_version"
        echo "#"

        idx=1
        while [ $idx -le $VBOX_USER_N ]
        do
            eval active='$VBOX_USER_'$idx'_ACTIVE'

            if [ "$active" = "yes" ]
            then
                eval username='$VBOX_USER_'$idx'_NAME'

                echo "$username:    root"
            fi

            idx=`expr $idx + 1`
        done
    } > $generate_aliases

    /var/install/config.d/mail.sh alias
}

#----------------------------------------------------------------------------------------
# generate vboxd.conf
#----------------------------------------------------------------------------------------
generate_vboxd_conf ()
{
    mecho "generating vboxd.conf ..."

    {
        echo "#--------------------------------------------------------------------------"
        echo "# ${generate_vboxd} file generated by ${pgmname} ${vbox_version}"
        echo "#"
        echo "# Do not edit this file, edit ${vboxfile}"
        echo "# Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#--------------------------------------------------------------------------"
      # echo "#"
        echo "# Login access list"
      # echo "#"
      # echo "# All hosts in the login access list (begins with 'L') are checked at"
      # echo "# login (server startup) time. If access is 'yes' the host can login"
      # echo "# and count messages without special access."
        echo
        echo "L:localhost:Y"

        idx=1
        while [ ${idx} -le ${VBOX_TRUSTED_NETS_N} ]
        do
            eval network='$VBOX_TRUSTED_NETS_'${idx}
            dnsnet=`netcalc dnsnet ${network}`

            echo "L:${dnsnet}`check_3point ${dnsnet}`*:Y"

            idx=`expr ${idx} + 1`
        done

        echo "L:*:N"
        echo
        echo "# Full access list"
      # echo "#"
      # echo "# All hosts in the full access list (begins with 'A') are checked if the"
      # echo "# server gets the 'login' command."
        echo

        # write full access list
        idx=1
        while [ ${idx} -le ${VBOX_USER_N} ]
        do
            eval active='$VBOX_USER_'${idx}'_ACTIVE'

            if [ "${active}" = "yes" ]
            then
                eval username='$VBOX_USER_'${idx}'_NAME'
                eval password='$VBOX_USER_'${idx}'_PASS'

                echo "A:localhost:RW:${username}:${password}:${vbox_spooldir}/${username}:incoming"

                jdx=1
                while [ ${jdx} -le ${VBOX_TRUSTED_NETS_N} ]
                do
                    eval network='$VBOX_TRUSTED_NETS_'${jdx}
                    dnsnet=`netcalc dnsnet ${network}`

                    echo "A:${dnsnet}`check_3point ${dnsnet}`*:RW:${username}:${password}:${vbox_spooldir}/${username}:incoming"

                    jdx=`expr ${jdx} + 1`
                done
            fi

            idx=`expr ${idx} + 1`
        done

        echo "A:*:!:!:!:!:!"
    } > ${generate_vboxd}
}

#----------------------------------------------------------------------------------------
# create /etc/xinetd.d/vbox
# $1 - quiet
#----------------------------------------------------------------------------------------
create_xinetd ()
{
    if [ "$1" != "-quiet" ]
    then
        mecho "creating xinet configuration file ..."
    fi

    {
        echo "#------------------------------------------------------------------"
        echo "# $generate_xinet_vbox file generated by $pgmname $vbox_version"
        echo "#"
        echo "# Do not edit this file, edit $vboxfile"
        echo "# Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#------------------------------------------------------------------"
        echo
        echo "service vboxd"
        echo "{"
        echo "  socket_type = stream"
        echo "  protocol = tcp"
        echo "  wait = no"
        echo "  user = root"
        echo "  server = /usr/local/vbox/sbin/vboxd"
        echo "  disable = no"
        echo "}"
    } > $generate_xinet_vbox
}

#----------------------------------------------------------------------------------------
# create /etc/logrotate.d/vbox
#----------------------------------------------------------------------------------------
create_logrotate ()
{
    mecho "creating logrotate configuration file ..."

    {
        echo "#------------------------------------------------------------------"
        echo "# ${generate_logrotate_vbox} file generated by ${pgmname} ${vbox_version}"
        echo "#"
        echo "# Do not edit this file, edit ${vboxfile}"
        echo "# Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#------------------------------------------------------------------"
        echo

        call_flag=0

        if [ "${START_VBOX}" = "yes" ]
        then
            if check_active_users
            then
                # active user exists, go on...
                idx=1
                while [ ${idx} -le ${VBOX_USER_N} ]
                do
                    eval active='$VBOX_USER_'${idx}'_ACTIVE'

                    if [ "${active}" = "yes" ]
                    then
                        # get port name
                        eval vboxport='$VBOX_USER_'${idx}'_ISDN_PORT'
                        vboxport=`echo ${vboxport}|sed 's#/dev/##g'`

                        printf "/var/log/vboxgetty-${vboxport}.log "

                        # check if call function has been enabled
                        eval call_active='$VBOX_USER_'${idx}'_CALL_ENABLED'

                        if [ ${call_flag} -eq 0 -a "${call_active}" = "yes" ]
                        then
                            call_flag=1
                        fi
                    fi

                    idx=`expr ${idx} + 1`
                done

                if [ ${call_flag} -eq 1 ]
                then
                    # add callback logfile to rotate list
                    printf "/var/log/vboxputty-ttyI15.log "
                fi

                echo "{"
                echo "    rotate ${VBOX_LOG_COUNT}"
                echo "    ${VBOX_LOG_INTERVAL}"
                echo "    compress"
                echo "    missingok"
                echo "    notifempty"
                echo "    sharedscripts"
                echo "    postrotate"
                echo "        /etc/init.d/vbox -quiet reload"
                echo "    endscript"
                echo "    }"
                echo
                echo "${configlog_file} {"
                echo "    rotate ${VBOX_LOG_COUNT}"
                echo "    size=10k"
                echo "    compress"
                echo "    missingok"
                echo "    notifempty"
                echo "    }"
                echo
            else
                echo "# No active VBOX users!"
            fi
        else
            echo "# VBOX disabled!"
        fi
    } > ${generate_logrotate_vbox}
}

#----------------------------------------------------------------------------------------
# generate vboxgetty.conf
#----------------------------------------------------------------------------------------
create_vboxgetty_conf ()
{
    echo "generating vboxgetty.conf ..."

    {
        echo "#--------------------------------------------------------------------------"
        echo "# $generate_vboxgetty file generated by $pgmname $vbox_version"
        echo "#"
        echo "# Do not edit this file, edit $vboxfile"
        echo "# Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#--------------------------------------------------------------------------"
        echo "# Global settings for all ports"

        # adpcm-2, adpcm-3, adpcm-4, ulaw
        if [ "$VBOX_COMPRESSION" = "" ]
        then
            VBOX_COMPRESSION='ulaw'
        fi

        echo "compression       `echo "$VBOX_COMPRESSION"|tr '[a-z]' '[A-Z]'`"

        # used file creatin mask
        if [ "$VBOX_UMASK" = "" ]
        then
            VBOX_UMASK="077"
        fi

        echo "umask             $VBOX_UMASK"

        echo "badinitsexit      10"
        echo "dropdtrtime       400"
        echo "initpause         2500"
        echo "commandtimeout    4"
        echo "echotimeout       4"
        echo "ringtimeout       6"
        echo "alivetimeout      1800"

        # if space < freespace a call isn't accepted
        echo "freespace         $VBOX_FREESPACE"

        # full debug: FEWIDJ
        echo "debuglevel        `echo "$VBOX_DEBUGLEVEL"|tr '[a-z]' '[A-Z]'`"
        echo

        ### write individual port settings ###
        idx=1
        activeflag=0
        while [ $idx -le $VBOX_USER_N ]
        do
            eval active='$VBOX_USER_'$idx'_ACTIVE'

            if [ "$active" = "yes" ]
            then
                eval port='$VBOX_USER_'$idx'_ISDN_PORT'
                eval modeminit='ATZ\&B512\&E$VBOX_USER_'$idx'_ISDN_MSN'
                eval username='$VBOX_USER_'$idx'_NAME'
                eval groupname='$VBOX_USER_'$idx'_GROUP'
                eval spooldir='$vbox_spooldir/$username'

                echo "# Settings for port $port"
                echo
                echo "port          $port"
                echo "modeminit     $modeminit"
                echo "user          $username"
                echo "group         $groupname"
                echo "spooldir      $spooldir"
                echo

                activeflag=1
            fi

            idx=`expr $idx + 1`
        done
    } > $generate_vboxgetty

    if [ $activeflag -eq 0 ]
    then
        mecho --error "Calls will not be answered because no active VBox found!"
    fi
}

#----------------------------------------------------------------------------------------
# generate vboxputty.conf
#----------------------------------------------------------------------------------------
create_vboxputty_conf ()
{
    idx=1
    while [ ${idx} -le ${VBOX_USER_N} ]
    do
        eval active='$VBOX_USER_'${idx}'_ACTIVE'
        eval call_enabled='$VBOX_USER_'${idx}'_CALL_ENABLED'

        eval port='/dev/ttyI15'
        eval modeminit='ATZ\&B512\&E$VBOX_USER_'${idx}'_ISDN_MSN'
        eval username='$VBOX_USER_'${idx}'_NAME'
        eval groupname='$VBOX_USER_'${idx}'_GROUP'

        generate_vboxputty=${vbox_spooldir}/${username}/vboxputty.conf

        if [ "${active}" = "yes" -a "${call_enabled}" = "yes" ]
        then
            mecho "generating vboxputty.conf file for user '${username}' ..."

            {
                #--------------------------------------------------------------------------
                print_short_header ${generate_vboxputty} ${pgmname} ${vbox} ${vbox_version}
                #--------------------------------------------------------------------------
                echo "# Global settings for all ports"

                # adpcm-2, adpcm-3, adpcm-4, ulaw
                if [ "${VBOX_COMPRESSION}" = "" ]
                then
                    VBOX_COMPRESSION='ulaw'
                fi

                echo "compression       `echo "${VBOX_COMPRESSION}"|tr '[a-z]' '[A-Z]'`"

                # used file creatin mask
                if [ "${VBOX_UMASK}" = "" ]
                then
                    VBOX_UMASK="077"
                fi

                echo "umask             ${VBOX_UMASK}"

                echo "badinitsexit      10"
                echo "dropdtrtime       400"
                echo "initpause         2500"
                echo "commandtimeout    4"
                echo "echotimeout       4"
                echo "ringtimeout       6"
                echo "alivetimeout      1800"

                # if space < freespace a call isn't accepted
                echo "freespace         ${VBOX_FREESPACE}"

                # full debug: FEWIDJ
                echo "debuglevel        `echo "${VBOX_DEBUGLEVEL}"|tr '[a-z]' '[A-Z]'`"
                echo

                ### write individual port settings ###

                echo "# Settings for port ${port}"
                echo "port          ${port}"
                echo "modeminit     ${modeminit}"
                echo "user          ${username}"
                echo "group         ${groupname}"
                echo "spooldir      ${vbox_spooldir}/${username}"
                echo
            } > ${generate_vboxputty}

            if [ -f ${vbox_spooldir}/${username}/vbox-nocallback ]
            then
               # callbacks have manually been disabled
               write_to_config_log -warn "Although parameter VBOX_USER_${idx}_CALL_ENABLED='yes' has been set"
               write_to_config_log -ff -warn "no callbacks will be forced because they've been disabled manually."
               write_to_config_log -ff -warn "Please use function 'Toggle VBox callback' to enable it!"
            fi
        else
            # remove obsolete config file
            rm -f ${generate_vboxputty}
        fi

        idx=`expr ${idx} + 1`
    done
}

#----------------------------------------------------------------------------------------
# remove vboxgetty call from /etc/inittab
# $1 - quiet
#----------------------------------------------------------------------------------------
remove_vboxgetty_from_inittab ()
{
    if [ "$1" != "-quiet" ]
    then
        mecho "removing vboxgetty entry from /etc/inittab ..."
    fi

    if [ -f $generate_inittab ]
    then
        rm -f $generate_inittab
        /var/install/bin/update-inittab $pgmname
    fi
}

#----------------------------------------------------------------------------------------
# add vboxgetty call to /etc/inittab
# $1 - quiet
#----------------------------------------------------------------------------------------
add_vboxgetty_to_inittab ()
{
    if [ "$1" != "-quiet" ]
    then
        mecho "adding vboxgetty entry to /etc/inittab ..."
    fi

    {
        echo "#"
        echo "# $generate_inittab list file generated by $pgmname $vbox_version"
        echo "#"

        idx=1
        while [ $idx -le $VBOX_USER_N ]
        do
            eval active='$VBOX_USER_'$idx'_ACTIVE'

            if [ "$active" = "yes" ]
            then
                eval port='$VBOX_USER_'$idx'_ISDN_PORT'
                eval iport=`echo $port|cut -f 2 -d "y"`

                echo "$iport:2345:respawn:/usr/local/vbox/sbin/vboxgetty -d $port"
            fi

            idx=`expr $idx + 1`
        done
    } > $generate_inittab

   /var/install/bin/update-inittab $pgmname
}

#----------------------------------------------------------------------------------------
# send voicemail as message
#----------------------------------------------------------------------------------------
create_vboxmail ()
{
    case ${MAIL_INSTALLED} in
        'vmail')
            # vmail package installed
            . ${vmailfile}

            qdomain="`/var/install/config.d/vmail.sh getdomain`"
            ;;
        'ssmtp')
            # this includes ssmtp
            . ${basefile}
            . ${ssmtpfile}

            if [ "${rewriteDomain}" != "" ]
            then
                # use rewriteDomain if set
                qdomain="${rewriteDomain}"
            else
                # use default domain
                qdomain="${DOMAIN_NAME}"
            fi
            ;;
        *)
            # mail package installed
            . ${mailfile}

            qdomain="${SMTP_QUALIFY_DOMAIN}"
            ;;
    esac

    idx=1
    while [ ${idx} -le ${VBOX_USER_N} ]
    do
        eval active='$VBOX_USER_'${idx}'_ACTIVE'

        if [ "${active}" = "yes" ]
        then
            eval username='$VBOX_USER_'${idx}'_NAME'
            eval numprefix='$VBOX_USER_'${idx}'_NUMBER_PREFIX'

            eval call_use='$VBOX_USER_'${idx}'_CALL_ENABLED'
            eval call_number='$VBOX_USER_'${idx}'_CALL_NUMBER'
            eval call_unknown='$VBOX_USER_'${idx}'_CALLERID_UNKNOWN'
            eval call_lookup='$VBOX_USER_'${idx}'_CALLERID_LOOKUP'

            eval mail_use='$VBOX_USER_'${idx}'_MAIL_ENABLED'             # _SEND_MAIL
            eval mail_type='$VBOX_USER_'${idx}'_MAIL_TYPE'               # _SEND_MAIL
            eval mail_addr='$VBOX_USER_'${idx}'_MAIL_ADDR'               # _SEND_ADDR
            eval mail_subject='$VBOX_USER_'${idx}'_MAIL_SUBJECT'         # _SEND_SUBJECT
            eval mail_sndformat='$VBOX_USER_'${idx}'_MAIL_SOUND_FORMAT'  # _SEND_SOUND_FORMAT

            eval rss_use='$VBOX_USER_'${idx}'_RSS_ENABLED'
            eval rss_subject='$VBOX_USER_'${idx}'_RSS_SUBJECT'

            if [ "${call_lookup}" = "" ]
            then
                call_lookup='no'
            fi

            if [ "${mail_sndformat}" = "wav" ]
            then
                # check if sound converter exists
                if [ ! -f /usr/bin/sox ]
                then
                    # sox binary doesn't exist
                    write_to_config_log -warn "VBOX_USER_${idx}_MAIL_SOUND_FORMAT='wav' has been set but no"
                    write_to_config_log -ff -warn "/usr/bin/sox binary can be found, falling back to au-format!"
                    mail_sndformat='au'
                fi
            fi

            if [ "${mail_sndformat}" = "mp3" -o "${rss_use}" = "yes" ]
            then
                # check if sound converter exists
                if [ ! -f /usr/bin/lame ]
                then
                    # lame binary doesn't exist
                    write_to_config_log -warn "VBOX_USER_${idx}_MAIL_SOUND_FORMAT='mp3' has been set but no"
                    write_to_config_log -ff -warn "/usr/bin/lame binary can be found, falling back to au-format!"
                    mail_sndformat='au'
                fi
            fi

            #--- call -------------------------------------------------------------------
            if [ "${call_use}" = "yes" ]
            then
                generate_mailtype=${vbox_spooldir}/${username}/call_nbr

                # remove previous notification(s)
                rm -f ${vbox_spooldir}/${username}/call_nbr

                mecho "creating call notification for \"${username}\" ..."

                {
                    echo "#"
                    echo "# ${generate_mailtype} file generated by ${pgmname} ${vbox_version}"
                    echo "#"

                    echo "SMTP_QUALIFY_DOMAIN=\"${qdomain}\""
                    echo "VBOX_PREFIX=\"${numprefix}\""
                    echo "VBOX_SENDTO=\"${call_number}\""
                    echo "VBOX_SUBJECT=\"${mail_subject}\""
                    echo "VBOX_UNKNOWN=\"${call_unknown}\""
                    echo "VBOX_LOOKUP=\"${call_lookup}\""
                } > ${generate_mailtype}

                # set file ownership and access rights
                chown ${username}: ${generate_mailtype}
                chmod 0644 ${generate_mailtype}
            else
                # remove notification
                rm -f ${vbox_spooldir}/${username}/call_nbr
            fi

            #--- mail -------------------------------------------------------------------
            if [ "${mail_use}" = "yes" ]
            then
                # setting file name
                case ${mail_type} in
                    subject ) generate_mailtype=${vbox_spooldir}/${username}/mail_subj;;
                    mail )    generate_mailtype=${vbox_spooldir}/${username}/mail_msg ;;
                    attach )  generate_mailtype=${vbox_spooldir}/${username}/mail_att ;;
                    *)        generate_mailtype='';;
                esac

                if [ "${generate_mailtype}" != "" ]
                then
                    # remove previous notification(s)
                    rm -f ${vbox_spooldir}/${username}/mail_*

                    mecho "creating mail notification for \"${username}\" ..."

                    {
                        echo "#"
                        echo "# ${generate_mailtype} file generated by ${pgmname} ${vbox_version}"
                        echo "#"

                        echo "SMTP_QUALIFY_DOMAIN=\"${qdomain}\""
                        echo "VBOX_PREFIX=\"${numprefix}\""
                        echo "VBOX_SOUND=\"${mail_sndformat}\""
                        echo "VBOX_SENDTO=\"${mail_addr}\""
                        echo "VBOX_SUBJECT=\"${mail_subject}\""
                        echo "VBOX_UNKNOWN=\"${call_unknown}\""
                        echo "VBOX_LOOKUP=\"${call_lookup}\""
                    } > ${generate_mailtype}

                    # set file ownership and access rights
                    chown ${username}: ${generate_mailtype}
                    chmod 0644 ${generate_mailtype}
                fi
            else
                # remove notification
                rm -f ${vbox_spooldir}/${username}/mail_*
            fi

            #--- rss --------------------------------------------------------------------
            if [ "${rss_use}" = "yes" ]
            then
                generate_mailtype=${vbox_spooldir}/${username}/write_rss

                # remove previous notification(s)
                rm -f ${vbox_spooldir}/${username}/write_rss

                # check if rss directory structure exists
                rssdir=${APACHE2_DOCUMENT_ROOT}/vbox-rss/${username}

                if [ "${APACHE2_INSTALLED}" = "yes" ]
                then
                    if [ ! -d ${rssdir} ]
                    then
                        # create directory
                        echo "- ${rssdir}"
                        mkdir -p ${rssdir}
                    fi

                    # set ownership
                    chown -R ${username} ${rssdir}
                    chgrp -R nogroup     ${rssdir}

                    mecho "creating rss notification for \"${username}\" ..."

                    {
                        echo "#"
                        echo "# ${generate_mailtype} file generated by ${pgmname} ${vbox_version}"
                        echo "#"

                        # add rss specific parameters
                        if [ "${APACHE2_PORT}" = "80" ]
                        then
                            # http server name only
                            rssserver="http://${APACHE2_SERVER_NAME}/"
                        else
                            # add port to http server name
                            rssserver="http://${APACHE2_SERVER_NAME}:${APACHE2_PORT}/"
                        fi

                        echo "SMTP_QUALIFY_DOMAIN=\"${qdomain}\""
                        echo "VBOX_PREFIX=\"${numprefix}\""
                        echo "VBOX_SENDTO=\"${mail_addr}\""
                        echo "VBOX_SUBJECT=\"${rss_subject}\""
                        echo "VBOX_UNKNOWN=\"${call_unknown}\""
                        echo "VBOX_LOOKUP=\"${call_lookup}\""

                        echo "VBOX_RSSADMIN=\"${APACHE2_SERVER_ADMIN}\""
                        echo "VBOX_RSSURL=\"${rssserver}\""
                        echo "VBOX_RSSDIR=\"${rssdir}\""
                    } > ${generate_mailtype}

                    # set file ownership and access rights
                    chown ${username}: ${generate_mailtype}
                    chmod 0644 ${generate_mailtype}
                else
                    # remove directory and files if it exists
                    if [ -d ${rssdir} ]
                    then
                        rm -f -r ${rssdir}
                    fi

                    write_to_config_log -warn "VBOX_USER_${idx}_RSS_ENABLED='yes' has been set but no"
                    write_to_config_log -ff -warn "apache2 package has been enabled, function disabled!"
                fi
            else
                # remove notification
                rm -f ${vbox_spooldir}/${username}/write_rss
            fi
        fi

        idx=`expr ${idx} + 1`
    done
}

#----------------------------------------------------------------------------------------
# check if section exists
# input:  $1 - user index
#         $2 - section name
# return: 0 - section exists
#         1 - section doesn't exist
#----------------------------------------------------------------------------------------
check_section ()
{
    udx=$1
    section_name="`echo "$2"|tr '[a-z]' '[A-Z]'`"
    retval=1

    if [ $# -ge 2 ]
    then
        if [ "$section_name" != "STANDARD" ]
        then
            retval=1
            eval snbr='$VBOX_USER_'$idx'_SECTION_N'

            sdx=1
            while [ $sdx -le $snbr ]
            do
                eval compare_name='$VBOX_USER_'$udx'_SECTION_'$sdx'_NAME'
                compare_name="`echo "$compare_name"|tr '[a-z]' '[A-Z]'`"

                if [ "$section_name" = "$compare_name" ]
                then
                    # section name found => ok
                    retval=0
                    break
                fi

                sdx=`expr $sdx + 1`
            done
        else
            # STANDARD => ok
            retval=0
        fi
    else
        # error insufficient number ob parameters
        retval=1
    fi

    return $retval
}

#----------------------------------------------------------------------------------------
# generate vbox.conf
#----------------------------------------------------------------------------------------
generate_vbox_conf ()
{
    idx=1
    while [ $idx -le $VBOX_USER_N ]
    do
        eval active='$VBOX_USER_'$idx'_ACTIVE'

        if [ "$active" = "yes" ]
        then
            eval username='$VBOX_USER_'$idx'_NAME'
            eval groupname='$VBOX_USER_'$idx'_GROUP'
            eval spooldir='$vbox_spooldir/$username'
            eval generate_vbox='$spooldir/vbox.conf'

            mecho "generating vbox.conf file for user '${username}' ..."

            # create dirs
            mkdir -p $spooldir/incoming
            mkdir -p $spooldir/messages
            mkdir -p $spooldir/archive

            {
                echo "#--------------------------------------------------------------------------"
                echo "# $generate_vbox file generated by $pgmname $vbox_version"
                echo "#"
                echo "# Do not edit this file, edit $vboxfile"
                echo "# Creation Date: ${EISDATE} Time: ${EISTIME}"
                echo "#--------------------------------------------------------------------------"
                echo

                ### individual caller ids ###
                # Format: PATTERN SECTION REALNAME"

                echo "[CALLERIDS]"

                cidx=1
                section_not_found=0
                eval cidnumber='$VBOX_USER_'$idx'_CALLERID_N'
                eval secnumber='$VBOX_USER_'$idx'_SECTION_N'

                while [ $cidx -le $cidnumber ]
                do
                    eval pattern='$VBOX_USER_'$idx'_CALLERID_'$cidx'_PATTERN'
                    eval section='$VBOX_USER_'$idx'_CALLERID_'$cidx'_SECTION_NAME'
                    eval realname='$VBOX_USER_'$idx'_CALLERID_'$cidx'_REALNAME'

                    section=`echo "$section"|tr '[a-z]' '[A-Z]'`

                    echo "$pattern $section $realname"

                    # check only if section exists
                    if [ $secnumber -le 0 ]
                    then
                        if [ "$section" = "STANDARD" ]
                        then
                            # STANDARD section => ok
                            retval=0
                        else
                            # not STANDARD section => error
                            retval=1
                        fi
                    else
                        check_section $idx "$section"
                        retval=$?
                    fi

                    if [ $retval -ne 0 -a $section_not_found -eq 0 ]
                    then
                        # section not found
                        write_to_config_log -warn "Section \"$section\" for user \"$username\" could not be found, please check the configuration!"
                        section_not_found=1
                    fi

                    cidx=`expr $cidx + 1`
                done

                eval realname='$VBOX_USER_'$idx'_CALLERID_UNKNOWN'

                echo "* - $realname"
                echo

                ### ring definition ###
                # Format: TIME DAYS RINGS"

                echo "[RINGS]"

                # write individual rings
                ridx=1
                eval ringnumber='$VBOX_USER_'$idx'_RING_N'

                while [ $ridx -le $ringnumber ]
                do
                    eval ringtime='$VBOX_USER_'$idx'_RING_'$ridx'_TIME'
                    eval ringdays='$VBOX_USER_'$idx'_RING_'$ridx'_DAYS'
                    eval ringcount='$VBOX_USER_'$idx'_RING_'$ridx'_RINGS'

                    check_time_setting "$ringtime" "VBOX_USER_${idx}_RING_${ridx}_TIME"

                    ringdays=`echo "$ringdays"|tr '[a-z]' '[A-Z]'`

                    echo "$ringtime $ringdays $ringcount"

                    ridx=`expr $ridx + 1`

                    eval ringnumber='$VBOX_USER_'$idx'_RING_N'
                done

                eval def_ringcount='$VBOX_USER_'$idx'_RING_STANDARD_RINGS'

                echo "* * $def_ringcount"
                echo

                ### standard section ###
                # Format: TIME DAYS STANDARD RECTIME [FLAG] [...]
                echo "[STANDARD]"

                std_def_time='*'
                std_def_days='*'
                std_def_msgfile='standard.msg'
                eval std_def_rectime='$VBOX_USER_'$idx'_SECTION_STANDARD_DEF_RECTIME'
                eval std_def_rings='$VBOX_USER_'$idx'_SECTION_STANDARD_DEF_RINGS'
                eval std_def_tollrings='$VBOX_USER_'$idx'_SECTION_STANDARD_DEF_TOLLRINGS'

                sidx=1
                eval secnbr='$VBOX_USER_'$idx'_SECTION_STANDARD_N'

                while [ $sidx -le $secnbr ]
                do
                    eval std_time='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_TIME'
                    eval std_days='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_DAYS'
                    eval std_msgfile='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_MSGFILE'
                    eval std_rectime='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_RECTIME'
                    eval std_rings='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_RINGS'
                    eval std_tollrings='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_TOLLRINGS'
                    eval std_option='$VBOX_USER_'$idx'_SECTION_STANDARD_'$sidx'_OPTION'

                    check_time_setting "$std_time" "VBOX_USER_${idx}_SECTION_STANDARD_${sidx}_TIME"

                    std_days=`echo "$std_days"|tr '[a-z]' '[A-Z]'`

                    if [ "$std_time" = "" ]
                    then
                        std_time=$std_def_time
                        write_to_config_log -info "Standard time for user \"$username\" has not been set, using default of \"$std_def_time\"!"
                    fi

                    if [ "$std_days" = "" ]
                    then
                        std_days=$std_def_days
                        write_to_config_log -info "Standard days for user \"$username\" has not been set, using default of \"$std_def_days\"!"
                    fi

                    if [ "$std_rectime" = "" ]
                    then
                        std_rectime=$std_def_rectime
                        write_to_config_log -info "Standard recording time for user \"$username\" has not been set, using default of \"$std_def_rectime\"!"
                    fi

                    if [ -f $spooldir/messages/$std_msgfile ]
                    then
                        std_rings="RINGS=$std_rings"

                        if [ "$std_tollrings" != "" ]
                        then
                            std_tollrings="TOLLRINGS=$std_tollrings"
                        else
                            std_tollrings=''
                        fi

                        echo "$std_time $std_days $std_msgfile $std_rectime $std_rings $std_tollrings $std_option"
                    else
                        # error
                        write_to_config_log -warn "Standard message file \"messages/$std_msgfile\" for user \"$username\" could not be found, entry will be ignored!"
                    fi

                    sidx=`expr $sidx + 1`
                done

                if [ "$std_def_rings" != "" ]
                then
                    std_def_rings="RINGS=$std_def_rings"
                else
                    std_def_rings=''
                fi

                if [ "$std_def_tollrings" != "" ]
                then
                    std_def_tollrings="TOLLRINGS=$std_def_tollrings"
                else
                    std_def_tollrings=''
                fi

                echo "$std_def_time $std_def_days $std_def_msgfile $std_def_rectime $std_def_rings $std_def_tollrings"
                echo

                ### user section ###
                eval secnbr='$VBOX_USER_'$idx'_SECTION_N'

                sidx=1
                while [ $sidx -le $secnbr ]
                do
                    eval sec_name='$VBOX_USER_'$idx'_SECTION_'$sidx'_NAME'
                    eval entry_nbr='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_N'

                    sec_name=`echo "$sec_name"|tr '[a-z]' '[A-Z]'`

                    echo "[$sec_name]"

                    eidx=1
                    while [ $eidx -le $entry_nbr ]
                    do
                        eval entry_time='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_TIME'
                        eval entry_days='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_DAYS'
                        eval entry_msgfile='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_MSGFILE'
                        eval entry_rectime='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_RECTIME'
                        eval entry_rings='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_RINGS'
                        eval entry_tollrings='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_TOLLRINGS'
                        eval entry_option='$VBOX_USER_'$idx'_SECTION_'$sidx'_ENTRY_'$eidx'_OPTION'

                        check_time_setting "$entry_time" "VBOX_USER_${idx}_SECTION_${sidx}_ENTRY_${eidx}_TIME"

                        entry_days=`echo "$entry_days"|tr '[a-z]' '[A-Z]'`

                        if [ "$entry_time" = "" ]
                        then
                            entry_time=$std_def_time
                            write_to_config_log -info "User entry time for user \"$username\" has not been set, using default of \"$std_def_time\"!"
                        fi

                        if [ "$entry_days" = "" ]
                        then
                            entry_days=$std_def_days
                            write_to_config_log -info "User entry days for user \"$username\" has not been set, using default of \"$std_def_days\"!"
                        fi

                        if [ "$entry_rectime" = "" ]
                        then
                            entry_rectime=$std_def_rectime
                            write_to_config_log -info "User entry recording time for user \"$username\" has not been set, using default of \"$std_def_rectime\"!"
                        fi

                        if [ -f $spooldir/messages/$entry_msgfile ]
                        then
                            if [ "$entry_rings" != "" ]
                            then
                                entry_rings="RINGS=$entry_rings"
                            else
                                entry_rings=''
                            fi

                            if [ "$entry_tollrings" != "" ]
                            then
                                entry_tollrings="TOLLRINGS=$entry_tollrings"
                            else
                                entry_tollrings=''
                            fi

                            echo "$entry_time $entry_days $entry_msgfile $entry_rectime $entry_rings $entry_tollrings $entry_option"
                        else
                            # error
                            write_to_config_log -warn "User entry message file \"messages/$entry_msgfile\" for user \"$username\" could not be found, entry will be ignored!"
                        fi

                        eidx=`expr $eidx + 1`
                    done

                    sidx=`expr $sidx + 1`
                done
            } > $generate_vbox

            # copying tcl and message files
            if [ ! -f "$spooldir/standard.tcl" ]
            then
                mecho "- copying standard.tcl file to $spooldir ..."
                cp /usr/local/vbox/examples/standard.tcl.example $spooldir/standard.tcl
            fi

            if [ ! -f "$spooldir/messages/beep.msg" ]
            then
                mecho "- copying beep.msg file to $spooldir/messages ..."
                cp /usr/local/vbox/examples/beep.msg.example $spooldir/messages/beep.msg
            fi

            if [ ! -f "$spooldir/messages/standard.msg" ]
            then
                mecho "- copying standard.msg file to $spooldir/messages ..."
                cp /usr/local/vbox/examples/standard.msg.example $spooldir/messages/standard.msg
            fi

            if [ ! -f "$spooldir/messages/timeout.msg" ]
            then
                mecho "- copying timeout.msg file to $spooldir/messages ..."
                cp /usr/local/vbox/examples/timeout.msg.example $spooldir/messages/timeout.msg
            fi

            # set ownership and group
            chown -R $username $spooldir
            chgrp -R $groupname $spooldir
            chmod 700 $spooldir $spooldir/incoming $spooldir/messages $spooldir/archive
        fi

        idx=`expr $idx + 1`
    done
}

#----------------------------------------------------------------------------------------
# compare previous and current user lists
#----------------------------------------------------------------------------------------
compare_old_new_user_lists ()
{
    mecho "comparing user lists ..."

    idx=1
    while [ $idx -le $VBOX_USER_N ]
    do
        eval username='$VBOX_USER_'$idx'_NAME'

        # compare with previous user list
        if [ $VBOX_OLD_USER_N -ne 0 ]                         # old user list exists?
        then
            oidx=1
            while [ $oidx -le $VBOX_OLD_USER_N ]
            do
                eval oldusername='$VBOX_OLD_USER_'$oidx'_NAME'

                if [ "$username" = "$oldusername" ]
                then
                  # echo "==:$idx:$oidx:$username:$oldusername:"
                    eval "VBOX_USER_${idx}_CREATE=0"            # don't create it
                    eval "VBOX_OLD_USER_${oidx}_DELETE=0"       # don't delete it
                fi

                oidx=`expr $oidx + 1`
            done
        fi

        # check if user exists
        grep -q "^${username}:" /etc/passwd

        if [ $? -ne 0 ]
        then
            eval "VBOX_USER_${idx}_CREATE=1"                # create entry
        else
            eval "VBOX_USER_${idx}_CREATE=0"                # don't create entry
        fi

        idx=`expr $idx + 1`
    done
}

#----------------------------------------------------------------------------------------
# delete obsolete system users and group
#   /etc/group
#   /etc/passwd
#----------------------------------------------------------------------------------------
remove_obsolete_system_users_and_groups ()
{
    mecho "removing obsolete system users and groups ..."

    idx=1
    while [ $idx -le $VBOX_OLD_USER_N ]
    do
        eval username='$VBOX_OLD_USER_'$idx'_NAME'
        eval groupname='$VBOX_OLD_USER_'$idx'_GROUP'
        eval userflag='$VBOX_OLD_USER_'$idx'_DELETE'

        if [ $userflag -eq 1 ]                # delete only if marked for deletion
        then
            # delete user from /etc/passwd and /etc/shadow
          # /var/install/bin/remove-user $username no
          # mecho "- user '$username' removed."
          # mecho --info "- remember to remove the obsolete user directory '$vbox_spooldir/$username'."
            mecho --info "- user '$username' is no longer used by vbox. If this system account is not used for"
            mecho --info "  other purposes you should remove it via the systems 'User administration'."
            sleep 2

            # delete group from /etc/group (remove-group verifys that no other user is in group)
          # /var/install/bin/remove-group $groupname
          # mecho "- group '$groupname' removed."
      # else
      #     mecho "- user '$username':'$groupname' not in removal list - will not be deleted"
        fi

        idx=`expr $idx + 1`
    done
}

#----------------------------------------------------------------------------------------
# create missing system users and group
#   /etc/group
#   /etc/passwd
#----------------------------------------------------------------------------------------
create_system_users_and_groups ()
{
    mecho "creating system users and groups ..."

    idx=1
    while [ $idx -le $VBOX_USER_N ]
    do
        eval active='$VBOX_USER_'$idx'_ACTIVE'

        if [ "$active" = "yes" ]
        then
            eval username='$VBOX_USER_'$idx'_NAME'
            eval groupname='$VBOX_USER_'$idx'_GROUP'
            eval userflag='$VBOX_USER_'$idx'_CREATE'
            eval password='$VBOX_USER_'$idx'_PASS'

            encodepass="`openssl passwd -crypt "${password}"`"
            home="${vbox_spooldir}/${username}"
            shell=/bin/sh

          # echo "create:$username:$userflag:"

            # check if group exists
            grep -q "^$groupname:" /etc/group

            if [ $? -ne 0 ]
            then
                # group in /etc/group eintragen und group id ermitteln
                # mecho "- group '$groupname' created."
                /var/install/bin/add-group $groupname
            fi

            groupid=`grep $groupname /etc/group | cut -d: -f3`

            if [ $userflag -eq 1 ]
            then
                # user in /etc/passwd und /etc/shadow eintragen
              # mecho "- user '$username' created."
                /var/install/bin/add-user "$username" "$encodepass" "" "$groupid" "VBOX user" "$home" "$shell"
            fi
        fi

        idx=`expr $idx + 1`
    done
}

#----------------------------------------------------------------------------------------
# generate old user list
#----------------------------------------------------------------------------------------
create_old_userlist ()
{
    mecho "creating user list ..."

    {
        echo "#"
        echo "# $generate_userlist list file generated by $pgmname $vbox_version"
        echo "#"

        echo "VBOX_OLD_USER_N='$VBOX_USER_N'"

        idx=1
        while [ $idx -le $VBOX_USER_N ]
        do
            eval username='$VBOX_USER_'${idx}'_NAME'
            eval groupname='$VBOX_USER_'${idx}'_GROUP'

            echo "VBOX_OLD_USER_${idx}_NAME='$username'"
            echo "VBOX_OLD_USER_${idx}_GROUP='$groupname'"
            echo "VBOX_OLD_USER_${idx}_DELETE='1'"

            idx=`expr $idx + 1`
        done
    } > $generate_userlist
}

#========================================================================================
# main
#========================================================================================

noconfirm=''

if [ "$1" = "-noconfirm" ]
then
    noconfirm=noconfirm
    shift
fi

case "$1"
in
    add_inittab )
        add_vboxgetty_to_inittab -quiet
        ;;
    del_inittab )
        remove_vboxgetty_from_inittab -quiet
        ;;
    add_xinetd )
        create_xinetd -quiet
        killall -1 xinetd 2>/dev/null
        ;;
    del_xinetdi )
        rm -f $generate_xinet_vbox
        killall -1 xinetd 2>/dev/null
        ;;
    create_devices )
        create_isdn_devices >/dev/null
        ;;
    * )
        if [ "$START_VBOX" = "yes" ]
        then
            # generate all configuration files
            echo "version: $vbox_version"

            if check_installed_apache2 -quiet
            then
                # installed
                APACHE2_INSTALLED='yes'

                tmp_conf=${tmpdir}/vboxconf.$$
                grep -E "^APACHE2_PORT|^APACHE2_SERVER_NAME|^APACHE2_SERVER_ADMIN|^APACHE2_DOCUMENT_ROOT" ${apache2file} > ${tmp_conf}
                . ${tmp_conf}

                rm -f ${tmp_conf}
            else
                # not installed
                APACHE2_INSTALLED=''
            fi

            if check_installed_mail -quiet
            then
                # mail
                MAIL_INSTALLED='mail'

                if [ -f ${mailfile} ]
                then
                    # mail package found
                    . ${mailfile}
                fi
            elif check_installed_vmail -quiet
            then
                # vmail
                MAIL_INSTALLED='vmail'

                if [ -f ${vmailfile} ]
                then
                    # vmail package found
                    . ${vmailfile}
                fi
            elif check_installed_ssmtp -quiet
            then
                # ssmtp
                MAIL_INSTALLED='ssmtp'

                if [ -f ${ssmtpfile} ]
                then
                    # ssmtp package found
                    . ${ssmtpfile}
                fi
            else
                # error
                MAIL_INSTALLED=''
                write_to_config_log -error 'No activated "mail", "vmail" or "ssmtp" package found!'
            fi

            create_isdn_devices

            compare_old_new_user_lists
            remove_obsolete_system_users_and_groups
            create_system_users_and_groups
            create_old_userlist

            add_ports_to_services
          # generate_smtp_aliases
            create_logrotate
            create_xinetd

            generate_vbox_conf
            generate_vboxd_conf
            create_vboxgetty_conf
            create_vboxputty_conf
            create_vboxmail

            add_vboxgetty_to_inittab

            compress_config_log

            if [ "${noconfirm}" != "noconfirm" ]
            then
                display_config_log
            fi
        else
            mecho --warn "VBox has been disabled."

            remove_vboxgetty_from_inittab
        fi

        mecho "finished."
        ;;
esac

#========================================================================================
# end
#========================================================================================
exit 0
